@@ -54,12 +54,12 @@ static const GthMenuEntry fixed_menu_entries[] = {
 
 
 static const GthMenuEntry vfs_open_actions_entries[] = {
-	{ N_("Open Folder"), "win.go-to-container-from-catalog", "<Alt>end" },
+	{ N_("Open Folder"), "win.go-to-container-from-catalog" },
 };
 
 
 static const GthMenuEntry vfs_other_actions_entries[] = {
-	{ N_("Remove from Catalog"), "win.remove-from-catalog", "Delete" },
+	{ N_("Remove from Catalog"), "win.remove-from-catalog" },
 };
 
 
@@ -196,16 +196,14 @@ catalogs__gth_browser_construct_cb (GthBrowser *browser)
 
 
 void
-catalogs__gth_browser_update_sensitivity_cb (GthBrowser *browser)
+catalogs__gth_browser_selection_changed_cb (GthBrowser *browser,
+					    int         n_selected)
 {
 	BrowserData *data;
-	int          n_selected;
 
 	data = g_object_get_data (G_OBJECT (browser), BROWSER_DATA_KEY);
 	g_return_if_fail (data != NULL);
 
-	n_selected = gth_file_selection_get_n_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
-
 	gth_window_enable_action (GTH_WINDOW (browser), "add-to-catalog", n_selected > 0);
 	gth_window_enable_action (GTH_WINDOW (browser), "remove-from-catalog", (n_selected > 0) && GTH_IS_FILE_SOURCE_CATALOGS (gth_browser_get_location_source (browser)));
 	gth_window_enable_action (GTH_WINDOW (browser), "go-to-container-from-catalog", n_selected == 1);
@@ -250,6 +248,16 @@ catalogs__gth_catalog_load_from_data_cb (const void *buffer)
 }
 
 
+GthCatalog *
+catalogs__gth_catalog_new_for_uri_cb (const char *uri)
+{
+	if (g_str_has_suffix (uri, ".catalog") || g_str_has_suffix (uri, ".gqv"))
+		return gth_catalog_new ();
+	else
+		return NULL;
+}
+
+
 void
 catalogs__gth_browser_folder_tree_popup_before_cb (GthBrowser    *browser,
 						   GthFileSource *file_source,
@@ -323,14 +331,10 @@ organize_button_clicked_cb (GtkButton  *button,
 
 void
 catalogs__gth_browser_load_location_after_cb (GthBrowser   *browser,
-					      GthFileData  *location_data,
-					      const GError *error)
+					      GthFileData  *location_data)
 {
 	BrowserData *data;
 
-	if ((location_data == NULL) || (error != NULL))
-		return;
-
 	data = g_object_get_data (G_OBJECT (browser), BROWSER_DATA_KEY);
 
 	if (GTH_IS_FILE_SOURCE_CATALOGS (gth_browser_get_location_source (browser))) {
@@ -468,6 +472,8 @@ process_rename_data_list (gpointer user_data)
 		GError     *error = NULL;
 
 		catalog = gth_catalog_load_from_file (rename_data->location);
+		if (catalog == NULL)
+			continue;
 
 		for (scan_files = rename_data->files, scan_new_files = rename_data->new_files;
 		     scan_files && scan_new_files;
