@@ -265,7 +265,7 @@ dlg_image_wall (GthBrowser *browser,
 	else
 		s_value = _g_settings_get_uri (data->settings, PREF_IMAGE_WALL_DESTINATION);
 	if (s_value == NULL)
-		s_value = g_strdup (get_home_uri ());
+		s_value = g_strdup (_g_uri_get_home ());
 	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), s_value);
 	g_free (s_value);
 
