@@ -46,6 +46,8 @@ static const GActionEntry actions[] = {
 	{ "edit-paste", gth_browser_activate_edit_paste },
 	{ "trash", gth_browser_activate_trash },
 	{ "delete", gth_browser_activate_delete },
+	{ "remove-from-source", gth_browser_activate_remove_from_source },
+	{ "remove-from-source-permanently", gth_browser_activate_remove_from_source_permanently },
 	{ "rename", gth_browser_activate_rename },
 	{ "file-list-rename", gth_browser_activate_file_list_rename },
 	{ "duplicate", gth_browser_activate_duplicate },
@@ -62,21 +64,22 @@ static const GActionEntry actions[] = {
 	{ "folder-context-move-to", gth_browser_activate_folder_context_move_to },
 	{ "folder-context-trash", gth_browser_activate_folder_context_trash },
 	{ "folder-context-delete", gth_browser_activate_folder_context_delete },
+	{ "open-with-gimp", gth_browser_activate_open_with_gimp },
 	{ "open-with-application", gth_browser_activate_open_with_application, "i" }
 };
 
 
 static const GthMenuEntry fixed_menu_entries_edit[] = {
-	{ N_("Cut"), "win.edit-cut", "<Control>x" },
-	{ N_("Copy"), "win.edit-copy", "<Control>c" },
-	{ N_("Paste"), "win.edit-paste", "<Control>v" },
+	{ N_("Cut"), "win.edit-cut" },
+	{ N_("Copy"), "win.edit-copy" },
+	{ N_("Paste"), "win.edit-paste" },
 };
 
 
 static const GthMenuEntry fixed_menu_entries_file[] = {
 	{ N_("Copy to…"), "win.copy-to-folder" },
 	{ N_("Move to…"), "win.move-to-folder" },
-	{ N_("Rename"), "win.file-list-rename", "F2" },
+	{ N_("Rename"), "win.file-list-rename" },
 };
 
 
@@ -112,16 +115,19 @@ static const GthMenuEntry folder_context_folder_entries[] = {
 
 
 static const GthMenuEntry vfs_entries[] = {
-	{ N_("Duplicate"), "win.duplicate", "<Control>d" }
+	{ N_("Duplicate"), "win.duplicate" }
 };
 
 
-static const GthAccelerator accelerators[] = {
-	{ "win.rename", "F2" },
-	{ "win.edit-cut", "<Control>x" },
-	{ "win.edit-copy", "<Control>c" },
-	{ "win.edit-paste", "<Control>v" },
-	{ "win.duplicate", "<Control>d" },
+static const GthShortcut shortcuts[] = {
+	{ "edit-cut", N_("Cut"), GTH_SHORTCUT_CONTEXT_BROWSER | GTH_SHORTCUT_CONTEXT_FIXED, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "<Primary>x" },
+	{ "edit-copy", N_("Copy"), GTH_SHORTCUT_CONTEXT_BROWSER | GTH_SHORTCUT_CONTEXT_FIXED, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "<Primary>c" },
+	{ "edit-paste", N_("Paste"), GTH_SHORTCUT_CONTEXT_BROWSER | GTH_SHORTCUT_CONTEXT_FIXED, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "<Primary>v" },
+	{ "rename", N_("Rename"), GTH_SHORTCUT_CONTEXT_BROWSER_VIEWER, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "F2" },
+	{ "duplicate", N_("Duplicate"), GTH_SHORTCUT_CONTEXT_BROWSER_VIEWER, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "<Primary>d" },
+	{ "remove-from-source", N_("Delete"), GTH_SHORTCUT_CONTEXT_BROWSER_VIEWER, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "Delete" },
+	{ "remove-from-source-permanently", N_("Delete permanently"), GTH_SHORTCUT_CONTEXT_BROWSER_VIEWER, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "<Shift>Delete" },
+	{ "open-with-gimp", N_("Open with Gimp"), GTH_SHORTCUT_CONTEXT_BROWSER_VIEWER, GTH_SHORTCUT_CATEGORY_FILE_MANAGER, "g" },
 };
 
 
@@ -261,7 +267,7 @@ gth_file_list_drag_data_received (GtkWidget        *file_view,
 
 			file_source = gth_browser_get_location_source (browser);
 			move = gdk_drag_context_get_selected_action (context) == GDK_ACTION_MOVE;
-			if (move && ! gth_file_source_can_cut (file_source, (GFile *) selected_files->data)) {
+			if (move && ! gth_file_source_can_cut (file_source)) {
 				GtkWidget *dialog;
 				int        response;
 
@@ -706,9 +712,9 @@ fm__gth_browser_construct_cb (GthBrowser *browser)
 	gth_menu_manager_append_entries (gth_browser_get_menu_manager (browser, GTH_BROWSER_MENU_MANAGER_FILE_FILE_ACTIONS),
 					 fixed_menu_entries_delete,
 				         G_N_ELEMENTS (fixed_menu_entries_delete));
-	gth_window_add_accelerators (GTH_WINDOW (browser),
-				     accelerators,
-				     G_N_ELEMENTS (accelerators));
+	gth_window_add_shortcuts (GTH_WINDOW (browser),
+				  shortcuts,
+				  G_N_ELEMENTS (shortcuts));
 
 	gth_browser_add_header_bar_button (browser,
 					   GTH_BROWSER_HEADER_SECTION_BROWSER_LOCATIONS,
@@ -809,8 +815,7 @@ fm__gth_browser_set_current_page_cb (GthBrowser *browser)
 
 void
 fm__gth_browser_load_location_after_cb (GthBrowser   *browser,
-					GthFileData  *location_data,
-					const GError *error)
+					GthFileData  *location_data)
 {
 	BrowserData    *data;
 	GtkWidget      *file_list;
@@ -820,9 +825,6 @@ fm__gth_browser_load_location_after_cb (GthBrowser   *browser,
 	int             n_source_targets;
 	GdkDragAction   source_actions;
 
-	if ((location_data == NULL) || (error != NULL))
-		return;
-
 	data = g_object_get_data (G_OBJECT (browser), BROWSER_DATA_KEY);
 	file_manager_update_ui (data, browser);
 
@@ -945,86 +947,101 @@ fm__gth_browser_folder_tree_drag_data_received_cb (GthBrowser    *browser,
 						   GList         *file_list,
 						   GdkDragAction  action)
 {
-	GthFileSource *file_source;
+	int            n_files;
+	GthFileSource *destination_source;
+	GFile         *first_file;
+	GthFileSource *file_list_source;
+	gboolean       move_files;
 	GtkWidget     *dialog;
 	GthTask       *task;
+	char          *message;
 	int            response;
 
 	if (destination == NULL)
 		return;
 
-	file_source = gth_main_get_file_source (destination->file);
-	if (file_source == NULL)
+	n_files = g_list_length (file_list);
+	if (n_files == 0)
 		return;
 
+	if ((action != GDK_ACTION_MOVE) && (action != GDK_ACTION_COPY))
+		return;
+
+	destination_source = gth_main_get_file_source (destination->file);
+	if (destination_source == NULL)
+		return;
+
+	first_file = G_FILE (file_list->data);
+	file_list_source = gth_main_get_file_source (first_file);
+	if (file_list_source == NULL)
+		return;
+
+	if (action == GDK_ACTION_MOVE)
+		action |= GDK_ACTION_COPY;
+
+	action = action & gth_file_source_get_drop_actions (destination_source, destination->file, first_file);
+	if (action == 0) {
+		_gtk_error_dialog_run (GTK_WINDOW (browser),
+				       "%s",
+				       _("Could not perform the operation"));
+		return;
+	}
+
+	move_files = (action & GDK_ACTION_MOVE) != 0;
+
 	/* ask confirmation */
 
 	response = GTK_RESPONSE_OK;
-	if ((action == GDK_ACTION_MOVE) || (action == GDK_ACTION_COPY)) {
-		int   n_files;
-		char *message;
+	if (n_files == 1) {
+		GFileInfo  *info;
+		char       *filename;
 
-		n_files = g_list_length (file_list);
-		g_return_if_fail (n_files >= 1);
+		info = gth_file_source_get_file_info (file_list_source, first_file, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME);
+		if (info != NULL)
+			filename = g_strdup (g_file_info_get_display_name (info));
+		else
+			filename = _g_file_get_display_name (first_file);
 
-		if (n_files == 1) {
-			char *filename = _g_file_get_display_name ((GFile *) file_list->data);
-			if (action == GDK_ACTION_MOVE)
-				message = g_strdup_printf (_("Do you want to move “%s” to “%s”?"), filename, g_file_info_get_display_name (destination->info));
-			else
-				message = g_strdup_printf (_("Do you want to copy “%s” to “%s”?"), filename, g_file_info_get_display_name (destination->info));
-			g_free (filename);
-		}
-		else {
-			if (action == GDK_ACTION_MOVE)
-				message = g_strdup_printf (_("Do you want to move the dragged files to “%s”?"), g_file_info_get_display_name (destination->info));
-			else
-				message = g_strdup_printf (_("Do you want to copy the dragged files to “%s”?"), g_file_info_get_display_name (destination->info));
-		}
-		dialog = _gtk_message_dialog_new (GTK_WINDOW (browser),
-						  GTK_DIALOG_MODAL,
-						  _GTK_ICON_NAME_DIALOG_QUESTION,
-						  message,
-						  NULL,
-						  _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
-						  ((action == GDK_ACTION_MOVE) ? _("Move") : _("_Copy")), GTK_RESPONSE_OK,
-						  NULL);
-		response = gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		if (move_files)
+			message = g_strdup_printf (_("Do you want to move “%s” to “%s”?"), filename, g_file_info_get_display_name (destination->info));
+		else
+			message = g_strdup_printf (_("Do you want to copy “%s” to “%s”?"), filename, g_file_info_get_display_name (destination->info));
 
-		g_free (message);
+		g_free (filename);
+		_g_object_unref (info);
 	}
+	else {
+		if (move_files)
+			message = g_strdup_printf (_("Do you want to move the dragged files to “%s”?"), g_file_info_get_display_name (destination->info));
+		else
+			message = g_strdup_printf (_("Do you want to copy the dragged files to “%s”?"), g_file_info_get_display_name (destination->info));
+	}
+	dialog = _gtk_message_dialog_new (GTK_WINDOW (browser),
+					  GTK_DIALOG_MODAL,
+					  _GTK_ICON_NAME_DIALOG_QUESTION,
+					  message,
+					  NULL,
+					  _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
+					  (move_files ? _("Move") : _("_Copy")), GTK_RESPONSE_OK,
+					  NULL);
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	g_free (message);
 
 	if (response != GTK_RESPONSE_OK)
 		return;
 
-	if ((action == GDK_ACTION_MOVE) && ! gth_file_source_can_cut (file_source, (GFile *) file_list->data)) {
-		dialog = _gtk_message_dialog_new (GTK_WINDOW (browser),
-						  GTK_DIALOG_MODAL,
-						  _GTK_ICON_NAME_DIALOG_QUESTION,
-						  _("Could not move the files"),
-						  _("Files cannot be moved to the current location, as alternative you can choose to copy them."),
-						  _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
-						  _GTK_LABEL_COPY, GTK_RESPONSE_OK,
-						  NULL);
-		response = gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+	/* exec task */
 
-		if (response == GTK_RESPONSE_CANCEL)
-			return;
-
-		action = GDK_ACTION_COPY;
-	}
-
-	task = gth_copy_task_new (file_source,
+	task = gth_copy_task_new (destination_source,
 				  destination,
-				  (action == GDK_ACTION_MOVE),
+				  move_files,
 				  file_list,
 				  -1);
 	gth_browser_exec_task (browser, task, GTH_TASK_FLAGS_DEFAULT);
 
 	g_object_unref (task);
-	g_object_unref (file_source);
+	g_object_unref (destination_source);
 }
 
 
@@ -1089,7 +1106,6 @@ fm__gth_browser_update_sensitivity_cb (GthBrowser *browser)
 	BrowserData   *data;
 	GthFileSource *file_source;
 	int            n_selected;
-	GthFileData   *location_data;
 	gboolean       sensitive;
 	GthFileData   *folder;
 
@@ -1099,8 +1115,7 @@ fm__gth_browser_update_sensitivity_cb (GthBrowser *browser)
 	file_source = gth_browser_get_location_source (browser);
 	n_selected = gth_file_selection_get_n_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 
-	location_data = gth_browser_get_location_data (browser);
-	sensitive = (n_selected > 0) && (file_source != NULL) && (location_data != NULL) && gth_file_source_can_cut (file_source, location_data->file);
+	sensitive = (n_selected > 0) && (file_source != NULL) && gth_file_source_can_cut (file_source);
 	gth_window_enable_action (GTH_WINDOW (browser), "edit-cut", sensitive);
 
 	sensitive = (n_selected > 0) && (file_source != NULL);
@@ -1157,80 +1172,3 @@ fm__gth_browser_unrealize_cb (GthBrowser *browser)
 	                                      G_CALLBACK (clipboard_owner_change_cb),
 	                                      browser);
 }
-
-
-gpointer
-fm__gth_browser_file_list_key_press_cb (GthBrowser  *browser,
-					GdkEventKey *event)
-{
-	gpointer  result = NULL;
-	guint     modifiers;
-	GList    *items;
-	GList    *file_data_list;
-	GList    *file_list;
-
-	modifiers = gtk_accelerator_get_default_mod_mask ();
-
-	switch (event->keyval) {
-	case GDK_KEY_g:
-		if ((event->state & modifiers) == 0) {
-			items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
-			file_data_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
-			file_list = gth_file_data_list_to_file_list (file_data_list);
-			_g_launch_command (GTK_WIDGET (browser), "gimp %U", "Gimp", file_list);
-
-			_g_object_list_unref (file_list);
-			_g_object_list_unref (file_data_list);
-			_gtk_tree_path_list_free (items);
-			result = GINT_TO_POINTER (1);
-		}
-		break;
-
-	case GDK_KEY_Delete:
-		if (((event->state & modifiers) == 0)
-		    || ((event->state & modifiers) == GDK_SHIFT_MASK)
-		    || ((event->state & modifiers) == GDK_CONTROL_MASK))
-		{
-			GthFileSource *source;
-			GthFileData   *location;
-
-			if ((event->state & modifiers) == 0) {
-				/* Removes the files from the current location,
-				 * for example: when viewing a catalog removes
-				 * the files from the catalog; when viewing a
-				 * folder removes the files from the folder. */
-				source = _g_object_ref (gth_browser_get_location_source (browser));
-				location = gth_browser_get_location_data (browser);
-			}
-			else {
-				/* When a key modifier is active, use the VFS
-				 * file source to delete the files from the
-				 * disk. */
-				source = gth_main_get_file_source_for_uri ("file:///");
-				location = NULL;
-			}
-
-			if (source == NULL)
-				return result;
-
-			items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
-			if (items == NULL)
-				return result;
-
-			file_data_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
-			gth_file_source_remove (source,
-						location,
-						file_data_list,
-						(event->state & modifiers) == GDK_SHIFT_MASK,
-						GTK_WINDOW (browser));
-			result = GINT_TO_POINTER (1);
-
-			_g_object_list_unref (file_data_list);
-			_gtk_tree_path_list_free (items);
-			_g_object_unref (source);
-		}
-		break;
-	}
-
-	return result;
-}
