@@ -21,7 +21,9 @@
 
 #include <config.h>
 #include <extensions/catalogs/gth-catalog.h>
+#ifdef HAVE_EXIV2
 #include <extensions/exiv2_tools/exiv2-utils.h>
+#endif
 #include <extensions/image_rotation/rotation-utils.h>
 #include "gth-import-task.h"
 #include "preferences.h"
@@ -579,12 +581,14 @@ file_buffer_ready_cb (void     **buffer,
 
 	file_data = self->priv->current->data;
 
+#ifdef HAVE_EXIV2
 	if (gth_main_extension_is_active ("exiv2_tools"))
 		exiv2_read_metadata_from_buffer (*buffer,
 						 count,
 						 file_data->info,
 						 TRUE,
 						 NULL);
+#endif
 
 	destination_file = get_destination_file (self, file_data);
 	if (destination_file == NULL)
