@@ -296,7 +296,7 @@ list_photos_ready_cb (GObject      *source_object,
 	}
 	gth_file_list_set_files (GTH_FILE_LIST (data->file_list), list);
 	update_selection_status (data);
-	gtk_widget_set_sensitive (GET_WIDGET ("download_button"), list != NULL);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, TRUE);
 
 	_g_object_list_unref (list);
 }
@@ -447,9 +447,7 @@ update_album_list (DialogData *data)
 		g_free (used_bytes);
 	}
 
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog),
-					   GTK_RESPONSE_OK,
-					   FALSE);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, FALSE);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("album_combobox")), -1);
 }
 
@@ -543,9 +541,22 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
 	data->builder = _gtk_builder_new_from_file ("import-from-picasaweb.ui", "picasaweb");
-	data->dialog = _gtk_builder_get_widget (data->builder, "import_dialog");
 	data->cancellable = g_cancellable_new ();
 
+	data->dialog = g_object_new (GTK_TYPE_DIALOG,
+				     "title", _("Import from Picasa Web Album"),
+				     "transient-for", GTK_WINDOW (browser),
+				     "modal", FALSE,
+				     "use-header-bar", _gtk_settings_get_dialogs_use_header (),
+				     NULL);
+	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (data->dialog))),
+			   _gtk_builder_get_widget (data->builder, "dialog_content"));
+	gtk_dialog_add_buttons (GTK_DIALOG (data->dialog),
+				_GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
+				_("_Import"), GTK_RESPONSE_OK,
+				NULL);
+	_gtk_dialog_add_class_to_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, GTK_STYLE_CLASS_SUGGESTED_ACTION);
+
 	{
 		GtkCellLayout   *cell_layout;
 		GtkCellRenderer *renderer;
@@ -590,7 +601,7 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (GET_WIDGET ("album_liststore")), ALBUM_NAME_COLUMN, GTK_SORT_ASCENDING);
 
-	gtk_widget_set_sensitive (GET_WIDGET ("download_button"), FALSE);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, FALSE);
 
 	data->preferences_dialog = gth_import_preferences_dialog_new ();
 	gtk_window_set_transient_for (GTK_WINDOW (data->preferences_dialog), GTK_WINDOW (data->dialog));
