@@ -25,5 +25,9 @@
 #include <gthumb.h>
 
 DEF_ACTION_CALLBACK (gth_browser_activate_slideshow)
+DEF_ACTION_CALLBACK (gth_slideshow_activate_close)
+DEF_ACTION_CALLBACK (gth_slideshow_activate_toggle_pause)
+DEF_ACTION_CALLBACK (gth_slideshow_activate_next_image)
+DEF_ACTION_CALLBACK (gth_slideshow_activate_previous_image)
 
 #endif /* ACTIONS_H */
