@@ -29,9 +29,6 @@
 #include "glib-utils.h"
 
 
-#define EPSILON ((ScaleReal) 1.0e-16)
-
-
 typedef double ScaleReal;
 typedef ScaleReal (*weight_func_t) (ScaleReal distance);
 
@@ -39,11 +36,7 @@ typedef ScaleReal (*weight_func_t) (ScaleReal distance);
 #ifdef HAVE_VECTOR_OPERATIONS
 
 
-typedef ScaleReal v4r __attribute__ ((vector_size(sizeof(ScaleReal)*4)));
-typedef union {
-	v4r       v;
-	ScaleReal r[4];
-} r4vector;
+typedef ScaleReal v4r __attribute__ ((vector_size(sizeof(ScaleReal) * 4)));
 
 
 #endif /* HAVE_VECTOR_OPERATIONS */
@@ -352,21 +345,12 @@ resize_filter_destroy (resize_filter_t *resize_filter)
 }
 
 
-static inline ScaleReal
-reciprocal (ScaleReal x)
-{
-	ScaleReal sign = x < 0.0 ? -1.0 : 1.0;
-	return (sign * x) >= EPSILON ? 1.0 / x : sign * (1.0 / EPSILON);
-}
-
-
 static void
 horizontal_scale_transpose (cairo_surface_t *image,
 			    cairo_surface_t *scaled,
 			    ScaleReal        scale_factor,
 			    resize_filter_t *resize_filter)
 {
-	ScaleReal  scale;
 	ScaleReal  support;
 	int        y;
 	int        image_width;
@@ -381,12 +365,9 @@ horizontal_scale_transpose (cairo_surface_t *image,
 	if (resize_filter->cancelled)
 		return;
 
-	scale = MAX ((ScaleReal) 1.0 / scale_factor + EPSILON, 1.0);
-	support = scale * resize_filter_get_support (resize_filter);
-	if (support < 0.5) {
+	support = (1.0 / scale_factor) * resize_filter_get_support (resize_filter);
+	if (support < 0.5)
 		support = 0.5;
-		scale = 1.0;
-	}
 
 	image_width = cairo_image_surface_get_width (image);
 	scaled_width = cairo_image_surface_get_width (scaled);
@@ -397,7 +378,6 @@ horizontal_scale_transpose (cairo_surface_t *image,
 	dest_rowstride = cairo_image_surface_get_stride (scaled);
 	weights = g_new (ScaleReal, 2.0 * support + 3.0);
 
-	scale = reciprocal (scale);
 	for (y = 0; y < scaled_height; y++) {
 	        guchar    *p_src_row;
 	        guchar    *p_dest_pixel;
@@ -410,7 +390,7 @@ horizontal_scale_transpose (cairo_surface_t *image,
 		int        i;
 		int        temp;
 #ifdef HAVE_VECTOR_OPERATIONS
-		r4vector   v_pixel, v_rgba;
+		v4r        v_pixel, v_rgba;
 #endif /* HAVE_VECTOR_OPERATIONS */
 
 		if (resize_filter->task != NULL) {
@@ -424,13 +404,15 @@ horizontal_scale_transpose (cairo_surface_t *image,
 			gth_async_task_set_data (resize_filter->task, NULL, NULL, &progress);
 		}
 
-		bisect = ((ScaleReal) y + 0.5) / scale_factor + EPSILON;
-		start = MAX (bisect - support + 0.5, 0);
-		stop = MIN (bisect + support + 0.5, (ScaleReal) image_width);
+		bisect = ((ScaleReal) y + 0.5) / scale_factor;
+		start = bisect - support + 0.5;
+		start = MAX (start, 0);
+		stop = bisect + support + 0.5;
+		stop = MIN (stop, image_width);
 
 		density = 0.0;
 		for (n = 0; n < stop - start; n++) {
-			weights[n] = resize_filter_get_weight (resize_filter, scale * ((ScaleReal) (start + n) - bisect + 0.5));
+			weights[n] = resize_filter_get_weight (resize_filter, scale_factor * ((ScaleReal) (start + n) - bisect + 0.5));
 			density += weights[n];
 		}
 
@@ -440,7 +422,7 @@ horizontal_scale_transpose (cairo_surface_t *image,
 		*/
 
 		if ((density != 0.0) && (density != 1.0)) {
-			density = reciprocal (density);
+			density = 1.0 / density;
 			for (i = 0; i < n; i++)
 				weights[i] *= density;
 		}
@@ -454,19 +436,18 @@ horizontal_scale_transpose (cairo_surface_t *image,
 
 #ifdef HAVE_VECTOR_OPERATIONS
 
-			v_rgba.v = (v4r) { 0.0, 0.0, 0.0, 0.0 };
+			v_rgba = (v4r) { 0.0, 0.0, 0.0, 0.0 };
 			for (i = 0; i < n; i++) {
-				v_pixel.v = (v4r) { p_src_pixel[0], p_src_pixel[1], p_src_pixel[2], p_src_pixel[3] };
-				v_rgba.v = v_rgba.v + (v_pixel.v * weights[i]);
-
+				v_pixel = (v4r) { p_src_pixel[0], p_src_pixel[1], p_src_pixel[2], p_src_pixel[3] };
+				v_rgba = v_rgba + (v_pixel * weights[i]);
 				p_src_pixel += 4;
 			}
-			v_rgba.v = v_rgba.v + 0.5;
+			v_rgba = v_rgba + 0.5;
 
-			p_dest_pixel[0] = CLAMP_PIXEL (v_rgba.r[0]);
-			p_dest_pixel[1] = CLAMP_PIXEL (v_rgba.r[1]);
-			p_dest_pixel[2] = CLAMP_PIXEL (v_rgba.r[2]);
-			p_dest_pixel[3] = CLAMP_PIXEL (v_rgba.r[3]);
+			p_dest_pixel[0] = CLAMP_PIXEL (v_rgba[0]);
+			p_dest_pixel[1] = CLAMP_PIXEL (v_rgba[1]);
+			p_dest_pixel[2] = CLAMP_PIXEL (v_rgba[2]);
+			p_dest_pixel[3] = CLAMP_PIXEL (v_rgba[3]);
 
 #else /* ! HAVE_VECTOR_OPERATIONS */
 
