@@ -219,25 +219,29 @@ static Format *
 get_format_from_extension (GthFileChooserDialog *self,
 			   const char           *filename)
 {
-	const char *ext;
+	char       *ext;
+	const char *only_ext;
 	GList      *scan;
 
-	ext = _g_uri_get_file_extension (filename);
+	ext = _g_uri_get_extension (filename);
 	if (ext == NULL)
 		return NULL;
 
-	if (ext[0] == '.')
-		ext++;
+	only_ext = (ext[0] == '.') ? ext + 1 : ext;
 
 	for (scan = self->priv->supported_formats; scan; scan = scan->next) {
 		Format *format = scan->data;
 		int     i;
 
 		for (i = 0; format->extensions[i] != NULL; i++)
-			if (g_ascii_strcasecmp (ext, format->extensions[i]) == 0)
+			if (g_ascii_strcasecmp (only_ext, format->extensions[i]) == 0) {
+				g_free (ext);
 				return format;
+			}
 	}
 
+	g_free (ext);
+
 	return NULL;
 }
 
