@@ -21,9 +21,11 @@
 
 #include <config.h>
 #include <glib/gi18n.h>
+#include "dlg-favorite-properties.h"
 #include "glib-utils.h"
 #include "gth-file-properties.h"
 #include "gth-main.h"
+#include "gth-preferences.h"
 #include "gth-property-view.h"
 #include "gth-string-list.h"
 #include "gth-time.h"
@@ -57,12 +59,18 @@ enum {
 
 
 struct _GthFilePropertiesPrivate {
+	GSettings     *settings;
 	GtkWidget     *main_container;
 	GtkWidget     *tree_view;
 	GtkListStore  *tree_model;
 	GtkWidget     *popup_menu;
+	GtkWidget     *copy_menu_item;
+	GtkWidget     *edit_favorites_menu_item;
+	GtkWidget     *context_menu_sep;
 	gboolean       show_details;
 	GthFileData   *last_file_data;
+	GHashTable    *favorites;
+	gboolean       default_favorites;
 };
 
 
@@ -162,29 +170,34 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 		if ((info->flags & GTH_METADATA_ALLOW_IN_PROPERTIES_VIEW) != GTH_METADATA_ALLOW_IN_PROPERTIES_VIEW)
 			continue;
 
+		if (info->id == NULL)
+			continue;
+
 		value = gth_file_data_get_attribute_as_string (file_data, info->id);
 		if ((value == NULL) || (*value == '\0')) {
 			g_free (value);
 			continue;
 		}
 
-		if (info->id != NULL) {
-			if (g_str_has_prefix (info->id, "Exif")) {
-				if (! self->priv->show_details)
-					continue;
-			}
-			else if (g_str_has_prefix (info->id, "Iptc")) {
-				if (! self->priv->show_details)
-					continue;
-			}
-			else if (g_str_has_prefix (info->id, "Xmp")) {
-				if (! self->priv->show_details)
-					continue;
-			}
-			else
-				if (self->priv->show_details)
-					continue;
+		if (! self->priv->show_details && ! self->priv->default_favorites) {
+			if (! g_hash_table_contains (self->priv->favorites, info->id))
+				continue;
 		}
+		else if (g_str_has_prefix (info->id, "Exif")) {
+			if (! self->priv->show_details)
+				continue;
+		}
+		else if (g_str_has_prefix (info->id, "Iptc")) {
+			if (! self->priv->show_details)
+				continue;
+		}
+		else if (g_str_has_prefix (info->id, "Xmp")) {
+			if (! self->priv->show_details)
+				continue;
+		}
+		else
+			if (self->priv->show_details)
+				continue;
 
 		if (value != NULL) {
 			char *utf8_value;
@@ -193,7 +206,7 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 			utf8_value = _g_utf8_from_any (value);
 			if (g_utf8_strlen (utf8_value, -1) > MAX_ATTRIBUTE_LENGTH)
 				g_utf8_strncpy (g_utf8_offset_to_pointer (utf8_value, MAX_ATTRIBUTE_LENGTH - 1), "…", 1);
-			tmp_value = _g_utf8_replace (utf8_value, "[\r\n]", " ");
+			tmp_value = _g_utf8_replace_pattern (utf8_value, "[\r\n]", " ");
 			g_free (value);
 			value = tmp_value;
 
@@ -263,11 +276,47 @@ gth_file_properties_finalize (GObject *base)
 	if (self->priv->popup_menu != NULL)
 		gtk_widget_destroy (self->priv->popup_menu);
 	_g_object_unref (self->priv->last_file_data);
+	_g_object_unref (self->priv->settings);
+	if (self->priv->favorites != NULL)
+		g_hash_table_unref (self->priv->favorites);
 
 	G_OBJECT_CLASS (gth_file_properties_parent_class)->finalize (base);
 }
 
 
+static void
+update_favorites (GthFileProperties *self)
+{
+	char *favorites;
+
+	if (self->priv->show_details)
+		return;
+
+	favorites = g_settings_get_string (self->priv->settings, PREF_BROWSER_FAVORITE_PROPERTIES);
+
+	if (self->priv->favorites != NULL)
+		g_hash_table_unref (self->priv->favorites);
+	self->priv->favorites = _g_str_split_as_hash_table (favorites);
+	self->priv->default_favorites = _g_str_equal (favorites, "default");
+
+	g_free (favorites);
+}
+
+
+static void
+update_context_menu_separator_visibility (GthFileProperties *self)
+{
+	int n_visible;
+
+	n_visible = 0;
+	if (gtk_widget_get_visible (self->priv->copy_menu_item))
+		n_visible++;
+	if (gtk_widget_get_visible (self->priv->edit_favorites_menu_item))
+		n_visible++;
+	gtk_widget_set_visible (self->priv->context_menu_sep, n_visible > 1);
+}
+
+
 static void
 gth_file_properties_set_property (GObject      *object,
 				  guint         property_id,
@@ -281,6 +330,11 @@ gth_file_properties_set_property (GObject      *object,
 	switch (property_id) {
 	case PROP_SHOW_DETAILS:
 		self->priv->show_details = g_value_get_boolean (value);
+		update_favorites (self);
+		if (self->priv->edit_favorites_menu_item != NULL) {
+			gtk_widget_set_visible (self->priv->edit_favorites_menu_item, ! self->priv->show_details);
+			update_context_menu_separator_visibility (self);
+		}
 		break;
 	default:
 		break;
@@ -353,6 +407,15 @@ copy_menu_item_activate_cb (GtkMenuItem *menuitem,
 }
 
 
+static void
+edit_favorites_menu_item_activate_cb (GtkMenuItem *menuitem,
+				      gpointer     user_data)
+{
+	GthFileProperties *self = user_data;
+	dlg_favorite_properties (GTH_BROWSER (_gtk_widget_get_toplevel_if_window (GTK_WIDGET (self))));
+}
+
+
 static gboolean
 tree_view_button_press_event_cb (GtkWidget      *widget,
 				 GdkEventButton *event,
@@ -362,21 +425,26 @@ tree_view_button_press_event_cb (GtkWidget      *widget,
 
 	if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) {
 		GtkTreePath *path;
+		gboolean     path_selected;
 
-		if (! gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (self->priv->tree_view),
-						     event->x,
-						     event->y,
-						     &path,
-						     NULL,
-						     NULL,
-						     NULL))
-			return FALSE;
+		path_selected = gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (self->priv->tree_view),
+			event->x,
+			event->y,
+			&path,
+			NULL,
+			NULL,
+			NULL);
+
+		gtk_widget_set_visible (self->priv->copy_menu_item, path_selected);
+		update_context_menu_separator_visibility (self);
+
+		if (path != NULL) {
+			gtk_tree_selection_select_path (GTK_TREE_SELECTION (gtk_tree_view_get_selection (GTK_TREE_VIEW (self->priv->tree_view))), path);
+			gtk_tree_path_free (path);
+		}
 
-		gtk_tree_selection_select_path (GTK_TREE_SELECTION (gtk_tree_view_get_selection (GTK_TREE_VIEW (self->priv->tree_view))), path);
 		gtk_menu_popup_at_pointer (GTK_MENU (self->priv->popup_menu), (GdkEvent *) event);
 
-		gtk_tree_path_free (path);
-
 		return TRUE;
 	}
 
@@ -389,11 +457,28 @@ tree_view_popup_menu_cb (GtkWidget *widget,
 			 gpointer   user_data)
 {
 	GthFileProperties *self = user_data;
+
+	gtk_widget_set_visible (self->priv->copy_menu_item, TRUE);
+	update_context_menu_separator_visibility (self);
+
 	gtk_menu_popup_at_pointer (GTK_MENU (self->priv->popup_menu), NULL);
+
 	return TRUE;
 }
 
 
+static void
+pref_favorite_properties_changed (GSettings  *settings,
+				  const char *key,
+				  gpointer    user_data)
+{
+	GthFileProperties *self = user_data;
+
+	update_favorites (self);
+	gth_property_view_set_file (GTH_PROPERTY_VIEW (self), self->priv->last_file_data);
+}
+
+
 static void
 gth_file_properties_init (GthFileProperties *self)
 {
@@ -405,6 +490,15 @@ gth_file_properties_init (GthFileProperties *self)
 	self->priv = gth_file_properties_get_instance_private (self);
 	self->priv->show_details = FALSE;
 	self->priv->last_file_data = NULL;
+	self->priv->settings = g_settings_new (GTHUMB_BROWSER_SCHEMA);
+	self->priv->favorites = NULL;
+
+	update_favorites (self);
+
+	g_signal_connect (self->priv->settings,
+			  "changed::" PREF_BROWSER_FAVORITE_PROPERTIES,
+			  G_CALLBACK (pref_favorite_properties_changed),
+			  self);
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
 	gtk_box_set_spacing (GTK_BOX (self), 6);
@@ -438,14 +532,29 @@ gth_file_properties_init (GthFileProperties *self)
 	/* popup menu */
 
 	self->priv->popup_menu = gtk_menu_new ();
-	menu_item = gtk_menu_item_new_with_label (_GTK_LABEL_COPY);
+
+	self->priv->copy_menu_item = menu_item = gtk_menu_item_new_with_label (_GTK_LABEL_COPY);
 	gtk_widget_show (menu_item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (self->priv->popup_menu),
-			       menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (self->priv->popup_menu), menu_item);
 	g_signal_connect (menu_item,
 			  "activate",
 			  G_CALLBACK (copy_menu_item_activate_cb),
 			  self);
+
+	self->priv->context_menu_sep = menu_item = gtk_separator_menu_item_new ();
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (self->priv->popup_menu), menu_item);
+
+	self->priv->edit_favorites_menu_item = menu_item = gtk_menu_item_new_with_label (_("Preferences"));
+	gtk_widget_set_visible (menu_item, ! self->priv->show_details);
+	gtk_menu_shell_append (GTK_MENU_SHELL (self->priv->popup_menu), menu_item);
+	g_signal_connect (menu_item,
+			  "activate",
+			  G_CALLBACK (edit_favorites_menu_item_activate_cb),
+			  self);
+
+	update_context_menu_separator_visibility (self);
+
 	g_signal_connect (self->priv->tree_view,
 			  "button-press-event",
 			  G_CALLBACK (tree_view_button_press_event_cb),
