@@ -703,22 +703,6 @@ compare_row_func (gconstpointer a,
 }
 
 
-static void
-_gth_file_store_sort (GthFileStore *file_store,
-		      gconstpointer pbase,
-		      gint          total_elems)
-{
-	if (file_store->priv->cmp_func == NULL)
-		return;
-
-	g_qsort_with_data (pbase,
-			   total_elems,
-			   (gsize) sizeof (GthFileRow *),
-			   compare_row_func,
-			   file_store);
-}
-
-
 static int
 compare_by_pos (gconstpointer a,
 		gconstpointer b,
@@ -736,6 +720,19 @@ compare_by_pos (gconstpointer a,
 }
 
 
+static void
+_gth_file_store_sort (GthFileStore *file_store,
+		      gconstpointer pbase,
+		      gint          total_elems)
+{
+	g_qsort_with_data (pbase,
+			   total_elems,
+			   (gsize) sizeof (GthFileRow *),
+			   (file_store->priv->cmp_func != NULL) ? compare_row_func : compare_by_pos,
+			   file_store);
+}
+
+
 static void
 _gth_file_store_compact_rows (GthFileStore *file_store)
 {
@@ -1124,16 +1121,21 @@ _gth_file_store_reorder (GthFileStore *file_store)
 }
 
 
+static void
+_gth_file_store_set_sort_func (GthFileStore        *file_store,
+			       GthFileDataCompFunc  cmp_func,
+			       gboolean             inverse_sort)
+{
+	file_store->priv->cmp_func = cmp_func;
+	file_store->priv->inverse_sort = inverse_sort;
+}
+
 void
 gth_file_store_set_sort_func (GthFileStore        *file_store,
 			      GthFileDataCompFunc  cmp_func,
 			      gboolean             inverse_sort)
 {
-	if ((cmp_func == file_store->priv->cmp_func) && (inverse_sort == file_store->priv->inverse_sort))
-		return;
-
-	file_store->priv->cmp_func = cmp_func;
-	file_store->priv->inverse_sort = inverse_sort;
+	_gth_file_store_set_sort_func (file_store, cmp_func, inverse_sort);
 	_gth_file_store_reorder (file_store);
 }
 
@@ -1660,6 +1662,8 @@ gth_file_store_reorder (GthFileStore *file_store,
 {
 	int i;
 
+	_gth_file_store_set_sort_func (file_store, NULL, FALSE);
+
 	for (i = 0; i < file_store->priv->num_rows; i++)
 		file_store->priv->rows[new_order[i]]->pos = i;
 
