@@ -25,6 +25,7 @@
 #include "dlg-preferences-browser.h"
 #include "dlg-preferences-extensions.h"
 #include "dlg-preferences-general.h"
+#include "dlg-preferences-shortcuts.h"
 #include "gth-file-comment.h"
 #include "gth-file-details.h"
 #include "gth-file-properties.h"
@@ -32,6 +33,16 @@
 #include "pixbuf-io.h"
 
 
+static GthShortcutCategory shortcut_categories[] = {
+	{ GTH_SHORTCUT_CATEGORY_HIDDEN, NULL, 0 },
+	{ GTH_SHORTCUT_CATEGORY_GENERAL, N_("General"), 31 },
+	{ GTH_SHORTCUT_CATEGORY_UI, N_("Show/Hide"), 30 },
+	{ GTH_SHORTCUT_CATEGORY_NAVIGATION, N_("Navigation"), 12 },
+	{ GTH_SHORTCUT_CATEGORY_FILE_MANAGER, N_("File Manager"), 10 },
+	{ GTH_SHORTCUT_CATEGORY_VIEWER, N_("Viewer"), 20 },
+};
+
+
 static void
 gth_main_register_default_file_loader (void)
 {
@@ -68,10 +79,12 @@ gth_main_register_default_types (void)
 	gth_main_register_type ("file-properties", GTH_TYPE_FILE_COMMENT);
 	gth_main_register_type ("file-properties", GTH_TYPE_FILE_DETAILS);
 	gth_main_register_default_file_loader ();
+	gth_main_register_shortcut_category (shortcut_categories, G_N_ELEMENTS (shortcut_categories));
 	gth_hook_add_callback ("dlg-preferences-construct", 1, G_CALLBACK (general__dlg_preferences_construct_cb), NULL);
 	gth_hook_add_callback ("dlg-preferences-apply", 1, G_CALLBACK (general__dlg_preferences_apply), NULL);
 	gth_hook_add_callback ("dlg-preferences-construct", 2, G_CALLBACK (browser__dlg_preferences_construct_cb), NULL);
 	gth_hook_add_callback ("dlg-preferences-apply", 2, G_CALLBACK (browser__dlg_preferences_apply), NULL);
+	gth_hook_add_callback ("dlg-preferences-construct", 50, G_CALLBACK (shortcuts__dlg_preferences_construct_cb), NULL);
 	gth_hook_add_callback ("dlg-preferences-construct", 9999, G_CALLBACK (extensions__dlg_preferences_construct_cb), NULL);
 	gth_hook_add_callback ("dlg-preferences-apply", 9999 /* Must be the last callback */, G_CALLBACK (extensions__dlg_preferences_apply), NULL);
 }
