@@ -27,11 +27,13 @@
 #include <gobject/gvaluecollector.h>
 #include "glib-utils.h"
 #include "gth-duplicable.h"
+#include "gth-file-source-vfs.h"
 #include "gth-filter.h"
 #include "gth-main.h"
 #include "gth-metadata-provider.h"
 #include "gth-user-dir.h"
 #include "gth-preferences.h"
+#include "gth-shortcut.h"
 #include "gtk-utils.h"
 #include "pixbuf-io.h"
 #include "typedefs.h"
@@ -105,6 +107,8 @@ struct _GthMainPrivate {
 	GPtrArray           *metadata_info;
 	GHashTable          *metadata_info_hash;
 	gboolean             metadata_info_sorted;
+	GPtrArray           *shortcut_category_v;
+	GHashTable          *shortcut_category_h;
 	GHashTable          *sort_types;
 	GHashTable          *image_loaders;
 	GHashTable          *types;
@@ -133,11 +137,14 @@ gth_main_finalize (GObject *object)
 	_g_object_list_unref (gth_main->priv->file_sources);
 
 	g_hash_table_unref (gth_main->priv->metadata_info_hash);
-	g_ptr_array_free (gth_main->priv->metadata_category, TRUE);
-	g_ptr_array_free (gth_main->priv->metadata_info, TRUE);
+	g_ptr_array_unref (gth_main->priv->metadata_category);
+	g_ptr_array_unref (gth_main->priv->metadata_info);
 	g_list_foreach (gth_main->priv->metadata_provider, (GFunc) g_object_unref, NULL);
 	g_list_free (gth_main->priv->metadata_provider);
 
+	g_ptr_array_unref (gth_main->priv->shortcut_category_v);
+	g_hash_table_unref (gth_main->priv->shortcut_category_h);
+
 	if (gth_main->priv->sort_types != NULL)
 		g_hash_table_unref (gth_main->priv->sort_types);
 	if (gth_main->priv->image_loaders != NULL)
@@ -182,6 +189,8 @@ gth_main_init (GthMain *main)
 	main->priv->metadata_info = g_ptr_array_new ();
 	main->priv->metadata_info_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 	main->priv->metadata_info_sorted = FALSE;
+	main->priv->shortcut_category_v = g_ptr_array_new ();
+	main->priv->shortcut_category_h = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 	main->priv->sort_types = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 	main->priv->image_loaders = g_hash_table_new_full (g_str_hash,
 						           (GEqualFunc) g_content_type_equals,
@@ -246,13 +255,8 @@ gth_main_get_file_source_for_uri (const char *uri)
 		}
 	}
 
-	if ((file_source == NULL) && ! g_str_has_prefix (uri, "vfs+")) {
-		char *vfs_uri;
-
-		vfs_uri = g_strconcat ("vfs+", uri, NULL);
-		file_source = gth_main_get_file_source_for_uri (vfs_uri);
-		g_free (vfs_uri);
-	}
+	if (file_source == NULL)
+		file_source = g_object_new (GTH_TYPE_FILE_SOURCE_VFS, NULL);
 
 	return file_source;
 }
@@ -274,6 +278,23 @@ gth_main_get_file_source (GFile *file)
 	return file_source;
 }
 
+
+char *
+gth_main_get_source_scheme (const char *uri)
+{
+	GList *scan;
+
+	for (scan = Main->priv->file_sources; scan; scan = scan->next) {
+		GthFileSource *registered_file_source = scan->data;
+
+		if (gth_file_source_supports_scheme (registered_file_source, uri))
+			return _g_uri_get_scheme (uri);
+	}
+
+	return g_strdup ("file");
+}
+
+
 GList *
 gth_main_get_all_file_sources (void)
 {
@@ -310,6 +331,56 @@ gth_main_get_all_entry_points (void)
 }
 
 
+GFile *
+gth_main_get_nearest_entry_point (GFile *file)
+{
+	GList *list;
+	GList *scan;
+	GList *entries;
+	char  *nearest_uri;
+	char  *uri;
+	int    file_uri_len;
+	int    min_diff;
+	GFile *nearest;
+
+	entries = NULL;
+	list = gth_main_get_all_entry_points ();
+	for (scan = list; scan; scan = scan->next) {
+		GthFileData *entry_point = scan->data;
+
+		if (_g_file_is_parent (entry_point->file, file))
+			entries = g_list_prepend (entries, g_file_get_uri (entry_point->file));
+	}
+
+	nearest_uri = NULL;
+	uri = g_file_get_uri (file);
+	file_uri_len = strlen (uri);
+	min_diff = 0;
+	for (scan = entries; scan; scan = scan->next) {
+		char *entry_uri = scan->data;
+		int   entry_len;
+		int   diff;
+
+		entry_len = strlen (entry_uri);
+		diff = abs (entry_len - file_uri_len);
+		if ((scan == entries) || (diff < min_diff)) {
+			min_diff = diff;
+			nearest_uri = entry_uri;
+		}
+	}
+	g_free (uri);
+
+	nearest = NULL;
+	if (nearest_uri != NULL)
+		nearest = g_file_new_for_uri (nearest_uri);
+
+	_g_string_list_free (entries);
+	_g_object_list_unref (list);
+
+	return nearest;
+}
+
+
 char *
 gth_main_get_gio_uri (const char *uri)
 {
@@ -584,6 +655,37 @@ gth_main_get_all_metadata_info (void)
 }
 
 
+GthShortcutCategory *
+gth_main_get_shortcut_category (const char *id)
+{
+	return g_hash_table_lookup (Main->priv->shortcut_category_h, id);
+}
+
+
+GPtrArray *
+gth_main_get_shortcut_categories (void)
+{
+	return Main->priv->shortcut_category_v;
+}
+
+
+void
+gth_main_register_shortcut_category (GthShortcutCategory *shortcut_category,
+				     int                  n_categories)
+{
+	int i;
+
+	g_mutex_lock (&register_mutex);
+	for (i = 0; i < n_categories; i++) {
+		if (gth_main_get_shortcut_category (shortcut_category[i].id) == NULL) {
+			g_ptr_array_add (Main->priv->shortcut_category_v, &shortcut_category[i]);
+			g_hash_table_insert (Main->priv->shortcut_category_h, shortcut_category[i].id, &shortcut_category[i]);
+		}
+	}
+	g_mutex_unlock (&register_mutex);
+}
+
+
 void
 gth_main_register_sort_type (GthFileDataSort *sort_type)
 {
@@ -934,7 +1036,7 @@ gth_main_get_type_set (const char *set_name)
 static void
 g_ptr_array_destroy (gpointer array)
 {
-	g_ptr_array_free ((GPtrArray *) array, TRUE);
+	g_ptr_array_unref ((GPtrArray *) array);
 }
 
 
@@ -1102,6 +1204,14 @@ gth_main_bookmarks_changed (void)
 }
 
 
+void
+gth_main_shortcuts_changed (GPtrArray *shortcuts_v)
+{
+	if (gth_shortcuts_write_to_file (shortcuts_v, NULL))
+		gth_monitor_shortcuts_changed (gth_main_get_default_monitor ());
+}
+
+
 GthFilterFile *
 gth_main_get_default_filter_file (void)
 {
@@ -1245,49 +1355,53 @@ gth_main_get_default_color_manager (void)
 void
 gth_main_activate_extensions (void)
 {
-	const char *mandatory_extensions[] = {	"file_viewer", /* keep the file viewer before any other viewer (see comment in gth-browser:file_metadata_ready_cb). */
+	const char *mandatory_extensions[] = {
+		"file_viewer", /* keep the file viewer before any other viewer (see comment in gth-browser:file_metadata_ready_cb). */
 #ifdef HAVE_LIBJPEG
-						"jpeg_utils",  /* mandatory if jpeg support is activated at compile time */
+		"jpeg_utils",  /* mandatory if jpeg support is activated at compile time */
 #endif
-						"cairo_io",
-						"image_viewer",
-						"file_tools",
-						NULL };
-	const char *default_extensions[] = {	"23hq",
-						"bookmarks",
-						"burn_disc",
-						"catalogs",
-						"change_date",
-						"comments",
-						"contact_sheet",
-						"convert_format",
-						"desktop_background",
-						"edit_metadata",
-						"exiv2_tools",
-						"facebook",
-						"file_manager",
-						"find_duplicates",
-						"flicker",
-						"gstreamer_tools",
-						"gstreamer_utils",
-						"image_print",
-						"image_rotation",
-						"importer",
-						"jpeg_utils",
-						"list_tools",
-						"oauth",
-						"photo_importer",
-						"photobucket",
-						"picasaweb",
-						"raw_files",
-						"red_eye_removal",
-						"rename_series",
-						"resize_images",
-						"search",
-						"selections",
-						"slideshow",
-						"webalbums",
-						NULL };
+		"cairo_io",
+		"image_viewer",
+		"file_tools",
+		NULL
+	};
+	const char *default_extensions[] = {
+		"23hq",
+		"bookmarks",
+		"burn_disc",
+		"catalogs",
+		"change_date",
+		"comments",
+		"contact_sheet",
+		"convert_format",
+		"desktop_background",
+		"edit_metadata",
+		"exiv2_tools",
+		"facebook",
+		"file_manager",
+		"find_duplicates",
+		"flicker",
+		"gstreamer_tools",
+		"gstreamer_utils",
+		"image_print",
+		"image_rotation",
+		"importer",
+		"jpeg_utils",
+		"list_tools",
+		"oauth",
+		"photo_importer",
+		"picasaweb",
+		"raw_files",
+		"red_eye_removal",
+		"rename_series",
+		"resize_images",
+		"search",
+		"selections",
+		"slideshow",
+		"terminal",
+		"webalbums",
+		NULL
+	};
 	GSettings            *settings;
 	char                **user_actived_extensions;
 	char                **actived_extensions;
