@@ -108,6 +108,8 @@ void            _gtk_dialog_add_to_window_group            (GtkDialog        *di
 void            _gtk_dialog_add_class_to_response          (GtkDialog        *dialog,
 		   	   	   	   	   	    int               respose_id,
 							    const char	     *class_name);
+void		_gtk_dialog_add_action_widget		   (GtkDialog        *dialog,
+							    GtkWidget        *button);
 GdkPixbuf *     _g_icon_get_pixbuf                         (GIcon            *icon,
 		 			    	    	    int               icon_size,
 		 			    	    	    GtkIconTheme     *icon_theme);
